<Query Kind="Statements" />

using WithParameterList;

var note = new Note (20, 100);

var note2 = note with { Duration = 200 };
note2.Dump ("Non-destructive mutation");

namespace WithParameterList
{
	record Note (int Pitch, int Duration);
}

namespace WithoutParameterList
{
	record Note
	{
		public int Pitch { get; init; }
		public int Duration { get; init; }

		public Note (int Pitch, int Duration)  // "Primary constructor"
		{
			this.Pitch = Pitch;
			this.Duration = Duration;
		}

		public void Deconstruct (out int Pitch, out int Duration)  // Deconstructor
		{
			Pitch = this.Pitch;
			Duration = this.Duration;
		}
	}
}

namespace FinalTranslation
{
	class Note
	{
		public int Pitch { get; init; }
		public int Duration { get; init; }

		public Note (int pitch, int duration)
		{
			Pitch = pitch;
			Duration = duration;
		}

		public void Deconstruct (out int Pitch, out int Duration)
		{
			Pitch = this.Pitch;
			Duration = this.Duration;
		}

		#region For non-destructive mutation

		protected Note (Note original)   // Copy constructor
		{
			Pitch = original.Pitch;
			Duration = original.Duration;
		}

		public virtual Note Clone() => new Note (this);

		#endregion

		#region For structural equality comparison

		public override bool Equals (object obj) =>
			Equals (obj as Note);

		public virtual bool Equals (Note other) =>
			other != null &&
			EqualityContract == other.EqualityContract &&
			Pitch == other.Pitch &&
			Duration == other.Duration;

		public static bool operator == (Note r1, Note r2) =>
			ReferenceEquals (r1, r2) || (r1?.Equals (r2) ?? false);

		public static bool operator != (Note r1, Note r2) =>
			!(r1 == r2);

		public override int GetHashCode() =>
			(EqualityContract, Pitch, Duration).GetHashCode();

		protected virtual Type EqualityContract => typeof (Note);

		#endregion

		#region ToString()

		public override string ToString()
		{
			var sb = new StringBuilder ("Note { ");
			if (PrintMembers (sb)) sb.Append (" ");
			sb.Append ("}");
			return sb.ToString();
		}

		protected virtual bool PrintMembers (StringBuilder builder)
		{
			builder.Append ("Pitch");
			builder.Append (" = ");
			builder.Append (Pitch.ToString());
			builder.Append (", ");
			builder.Append ("Duration");
			builder.Append (" = ");
			builder.Append (Duration.ToString());
			return true;
		}
		#endregion
	}
}